#include <QTimer>
#include <QDebug>
#include "StateMachine.h"
//*********************************************************************************************************************


StateMachine::StateMachine(QObject *parent) : QObject(parent)
{
    timerStateMachine = new QTimer();
    timerEvent1 = new QTimer();
    timerEvent2 = new QTimer();

    // вызов обработчика машины состояний по таймауту таймера
    connect(timerStateMachine, &QTimer::timeout, this, &StateMachine::StateMachineWork);
    connect(timerEvent1, &QTimer::timeout, this, &StateMachine::Event1Set);
    connect(timerEvent2, &QTimer::timeout, this, &StateMachine::Event2Set);
}
//*********************************************************************************************************************


StateMachine::~StateMachine()
{
    timerStateMachine->stop();
    timerEvent1->stop();
    timerEvent2->stop();
    delete timerStateMachine;
    delete timerEvent1;
    delete timerEvent2;
}
//*********************************************************************************************************************


// обработка состояний
void StateMachine::StateMachineWork(void)
{
        if (tableStateProcess[tablePointer].worker)             // если существует указатель на функцию
            (this->*tableStateProcess[tablePointer].worker)();  // выполнение функции

        if (event) {                                            // если в функции произошло событие
            tablePointer += event;                              // изменяется указатель на строку таблицы в
            event = 0;                                          // соответствии с числом, заданным в событии
        }
        else        // если указатель на строку таблицы машины состояний не был задан событием
            tablePointer += tableStateProcess[tablePointer].nextStep;   // к указателю на строку таблицы добавляется
                                                                        // число шагов из текущей строки таблицы
                                                                        // число шагов может быть отрицательным
                                                                        // для организации цикла
        currentState = tableStateProcess[tablePointer].newState;        // текущее состояние
#if DEBUG_STATE_MACHINE
            qDebug() << "===== Выполнен шаг по таблице машины состояний =====";
            qDebug() << "nextStep            =" << tableStateProcess[tablePointer].nextStep;
            qDebug() << "tablePointer        =" << tablePointer;
            qDebug() << "currentState        =" << currentState;
#endif
}
//*********************************************************************************************************************


// старт обработчика машины состояний
void StateMachine::StateMachineStart(void)
{
    for (quint8 i = 0; i < 50; ++ i)
        if (tableStateProcess[i].newState != 0) {
            tablePointer = i;
            break;
        }

    if (tablePointer) {
        TimerStateMachineStart(STATE_MACHINE_CYCLE);
        emit SetTextSignal(SYS_TIME + "Старт машины состояний");
        emit SetTextSignal("=================================================");
    }
    else {
        emit SetTextSignal(SYS_TIME + "Машина состояний не инициализирована");
        emit SetTextSignal("=================================================");
    }
}
//*********************************************************************************************************************


// останов обработчика машины состояний
void StateMachine::StateMachineStop(void)
{
    if (tablePointer) {
        timerStateMachine->stop();
        timerEvent1->stop();
        timerEvent2->stop();
        tablePointer = 0;
        event = 0;
        emit SetTextSignal(SYS_TIME + "Останов машины состояний");
        emit SetTextSignal("=================================================");
    }
    else {
        emit SetTextSignal(SYS_TIME + "Машина состояний не запущена");
        emit SetTextSignal("=================================================");
    }
}
//*********************************************************************************************************************


// загрузка рабочей таблицы состояний
void StateMachine::LoadTableProcess(transition *tabSt, quint64 size)
{
    if (tablePointer == 0) {
        for (quint8 i = 0; i < 50; ++ i)
            tableStateProcess[i] = {IDLE_STATE, NULL, WAIT};
        for (quint8 i = 0; i < size / sizeof(transition); ++ i)
            tableStateProcess[i] = tabSt[i];
        event = 0;
        emit SetTextSignal(SYS_TIME + "Загружена таблица состояний");
        emit SetTextSignal("=================================================");
    }
}
//*********************************************************************************************************************


void StateMachine::Event1Set(void)
{
    event = EVENT_1;
    emit SetTextSignal(SYS_TIME + "Произошло событие 1");
    emit SetTextSignal("=================================================");
}
//*********************************************************************************************************************


void StateMachine::Event2Set(void)
{
    event = EVENT_2;
    emit SetTextSignal(SYS_TIME + "Произошло событие 2");
    emit SetTextSignal("=================================================");
}
//*********************************************************************************************************************


void StateMachine::Function1(void)
{
    TimerEvent1Start(2000);
    emit SetTextSignal(SYS_TIME + "Вызвана функция 1");
    emit SetTextSignal("=================================================");
}
//*********************************************************************************************************************


void StateMachine::Function2(void)
{
    TimerEvent2Start(2000);
    emit SetTextSignal(SYS_TIME + "Вызвана функция 2");
    emit SetTextSignal("=================================================");
}
//*********************************************************************************************************************


void StateMachine::Function3(void)
{
    emit SetTextSignal(SYS_TIME + "Начало цикла");
    emit SetTextSignal("=================================================");
    emit SetTextSignal(SYS_TIME + "Вызвана функция 3");
    emit SetTextSignal("=================================================");
    event = EVENT_3;
    emit SetTextSignal(SYS_TIME + "Произошло событие 3");
    emit SetTextSignal("=================================================");
}
//*********************************************************************************************************************


void StateMachine::Function4(void)
{
	emit SetTextSignal(SYS_TIME + "Вызвана функция 4");
    emit SetTextSignal("=================================================");
	if (!(flagF4)) {
		event = EVENT_4;
		emit SetTextSignal(SYS_TIME + "Произошло событие 4");
        emit SetTextSignal("=================================================");
		flagF4 = true;
	}
	else
		flagF4 = false;
}
//*********************************************************************************************************************


void StateMachine::Function5(void)
{
    emit SetTextSignal(SYS_TIME + "Вызвана функция 5");
    emit SetTextSignal("=================================================");
    event = EVENT_5;
    emit SetTextSignal(SYS_TIME + "Произошло событие 5");
    emit SetTextSignal("=================================================");
}
//*********************************************************************************************************************


void StateMachine::Function6(void)
{
    emit SetTextSignal(SYS_TIME + "Вызвана функция 6");
    emit SetTextSignal("=================================================");
    emit SetTextSignal(SYS_TIME + "Возврат к началу цикла");
    emit SetTextSignal("=================================================");
	if (cycleEnd) {
		event = EVENT_6;
		emit SetTextSignal(SYS_TIME + "Произошло событие 6");
        emit SetTextSignal("=================================================");
		emit SetTextSignal(SYS_TIME + "Цикл прерван");
        emit SetTextSignal("=================================================");
        cycleEnd = false;
    }
}
//*********************************************************************************************************************


void StateMachine::Function7(void)
{
    emit SetTextSignal(SYS_TIME + "Вызвана функция 7");
    emit SetTextSignal("=================================================");
}
//*********************************************************************************************************************


void StateMachine::Function8(void)
{
    emit SetTextSignal(SYS_TIME + "Вызвана функция 8");
    emit SetTextSignal("=================================================");
}
//*********************************************************************************************************************


void StateMachine::FunctionEnd(void)
{
    emit SetTextSignal(SYS_TIME + "Закончено выполнение всех функций из таблицы состояний");
    emit SetTextSignal("=================================================");
    tablePointer = 0;
}
//*********************************************************************************************************************


void StateMachine::CycleEndSetSlot(void)
{
    if (tablePointer) {
        cycleEnd = true;
        emit SetTextSignal(SYS_TIME + "Цикл будет прерван");
        emit SetTextSignal("=================================================");
    }
    else {
        emit SetTextSignal(SYS_TIME + "Машина состояний не запущена");
        emit SetTextSignal("=================================================");
    }
}
//*********************************************************************************************************************
