﻿#ifndef STATEMACHINE_H
#define STATEMACHINE_H
//*********************************************************************************************************************


#include <QObject>
#include <QDateTime>
#include <QTimer>
//*********************************************************************************************************************


#define DEBUG_STATE_MACHINE         false //true //

#define SYS_TIME      QTime::currentTime().toString("HH:mm:ss:zzz   ")
//#define SYS_TIME      QDateTime::currentDateTime().toString("dd.MM.yy   hh:mm:ss   ");

#define STATE_MACHINE_CYCLE         2000       // мс, период вызова обработчика машины состояний

// события в программе, задают число шагов по таблице состояний
#define EVENT_1     1       // событие 1, 1 шаг вперед по таблице состояний
#define EVENT_2     1
#define EVENT_3     2
#define EVENT_4     3
#define EVENT_5     2
#define EVENT_6     1

// число шагов, задаваемое в таблице состояний
#define STEP_FORWARD_1  1   // переход к следующему состоянию
#define STEP_BACK_5     -5  // 5 шагов назад по таблице состояний
#define WAIT            0   // ожидание нового события или бездействие программы

// состояния программы
typedef enum {
    IDLE_STATE = 0,         // состояние бездействия

    STATE_1,                // состояние 1
    STATE_2,
    STATE_3,
    STATE_4,
    STATE_5,
    STATE_6,
    STATE_7,
    STATE_8,
    STATE_9,
    STATE_10,
    STATE_11,
    STATE_12,
    END_STATE,

    MAX_STATES              // максимальное число состояний

} stateMachineEnum;


class StateMachine  : public QObject
{
    Q_OBJECT
public:
    explicit StateMachine(QObject *parent = nullptr);
    ~StateMachine();

    void StateMachineStart(void);
    void StateMachineStop(void);

    typedef void (StateMachine::*callback) (void);

    typedef struct
    {
        stateMachineEnum newState;
        callback   worker;
        qint8      nextStep;
    } transition;

    void LoadTableProcess(transition *, quint64);

    void Function1(void);
    void Function2(void);
    void Function3(void);
    void Function4(void);
    void Function5(void);
    void Function6(void);
    void Function7(void);
    void Function8(void);
    void FunctionEnd(void);

private:
    transition tableStateProcess[50] = {};
    quint8 tablePointer = 0;

    QTimer *timerStateMachine;          // таймер периода вызова обработчика машины состояний
    QTimer *timerEvent1;                // таймер периода вызова события 1
    QTimer *timerEvent2;                // таймер периода вызова события 2

    quint8 currentState = IDLE_STATE;   // состояние при старте программы

    quint8 event = 0;                   // событие машины состояний

    bool cycleEnd = false, flagF4 = false;

    void TimerStateMachineStart(quint64);
    void TimerEvent1Start(quint64);
    void TimerEvent2Start(quint64);

    void StateMachineWork(void);

    void Event1Set(void);
    void Event2Set(void);

signals:
    void SetTextSignal(QString);

public slots:
    void CycleEndSetSlot(void);
};
//*********************************************************************************************************************


#endif // STATEMACHINE_H
//*********************************************************************************************************************
