#include <QGraphicsDropShadowEffect>
#include "mainwindow.h"
#include "ui_mainwindow.h"
//*********************************************************************************************************************


MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    stateMachine = new StateMachine(this);

    ui->groupBox->setTitle("");
    ui->gridLayout->addWidget(ui->pushButton_3, 0, 0, 1, 1);
    ui->gridLayout->addWidget(ui->pushButton_2, 0, 1, 1, 1);
    ui->gridLayout->addWidget(ui->pushButton_4, 0, 2, 1, 1);
    ui->gridLayout->addWidget(ui->pushButton, 0, 3, 1, 1);
    ui->gridLayout->addWidget(ui->textBrowser, 1, 0, 1, 4);
    ui->groupBox->setLayout(ui->gridLayout);

    QGraphicsDropShadowEffect *shadow = new QGraphicsDropShadowEffect;
    // QColor *color = new QColor("darkgray");
    QColor *color = new QColor("gray");
    // QColor *color = new QColor("lightgray");
    shadow->setColor(*color);
    shadow->setBlurRadius(5);
    shadow->setXOffset(5);
    shadow->setYOffset(5);
    ui->groupBox->setGraphicsEffect(shadow);

    connect(stateMachine, &StateMachine::SetTextSignal, this, &MainWindow::SetTextSlot);
    connect(this, &MainWindow::CycleEndSetSignal, stateMachine, &StateMachine::CycleEndSetSlot);

    // для отбражения иконки  в Linux
    // иконка из ресурса программы - :
    setWindowIcon(QIcon(":Radio.ico"));

    setWindowTitle("TableStateMachine");
}
//*********************************************************************************************************************


MainWindow::~MainWindow()
{
    delete stateMachine;
    delete ui;
}
//*********************************************************************************************************************


void MainWindow::SetTextSlot(QString str)
{
    ui->textBrowser->append(str);
}
//*********************************************************************************************************************


void MainWindow::on_pushButton_clicked()
{
    emit CycleEndSetSignal();
}
//*********************************************************************************************************************


void MainWindow::on_pushButton_2_clicked()
{
    stateMachine->StateMachineStart();
}
//*********************************************************************************************************************


void MainWindow::on_pushButton_3_clicked()
{
    LoadProcess();
}
//*********************************************************************************************************************


void MainWindow::on_pushButton_4_clicked()
{
    stateMachine->StateMachineStop();
}
//*********************************************************************************************************************


// установка значений для машины состояний
void MainWindow::LoadProcess(void)
{
    /***************************************************************************
        инициализация таблицы состояний, в каждой строке таблицы три значения:
        1.  имя текущего состояния
        2.  ссылка на функцию, которую небходимо выполнить,
            в некоторых функциях возможно ветвление
			с выполнением нескольких шагов вперед по таблице
        3.  число шагов машины состояний по таблице
            (вперед при положительных значениях, назад при отрицательных или 0 при ожидании событий)
    ***************************************************************************/

    StateMachine::transition tableStateInit[] = {
        {IDLE_STATE,    NULL,                           WAIT}
        // в функции запускается таймер, в обработчике таймаута которого
        // происходит событие, сдвигающее указатель строки таблицы на 1 шаг
        // вперед по таблице для перехода к строке с функцией Function2
        ,{STATE_1,      &StateMachine::Function1,       STEP_FORWARD_1}
        ,{STATE_2,      NULL,                           WAIT}
        // в функции запускается таймер, в обработчике таймаута которого
        // происходит событие, сдвигающее указатель строки таблицы на 1 шаг
        // вперед по таблице для перехода к строке с функцией Function3
        ,{STATE_3,      &StateMachine::Function2,       STEP_FORWARD_1}
        ,{STATE_4,      NULL,                           WAIT}
        // в функции происходит событие, сдвигающее указатель строки таблицы
        // на 2 шага вперед по таблице для перехода к строке с функцией Function4
        ,{STATE_5,      &StateMachine::Function3,       STEP_FORWARD_1} // начало цикла
        ,{STATE_6,      NULL,                           WAIT}
        // в функции может произойти событие, сдвигающее указатель строки таблицы
        // на 3 шага вперед по таблице для перехода к строке с функцией Function6
        ,{STATE_7,      &StateMachine::Function4,       STEP_FORWARD_1}
        // в функции происходит событие, сдвигающее указатель строки таблицы
        // на 2 шага вперед по таблице для перехода к строке с функцией Function4
        ,{STATE_8,      &StateMachine::Function5,       STEP_FORWARD_1}
        ,{STATE_9,      NULL,                           WAIT}
        // в функции может произойти событие, сдвигающее указатель строки
        // таблицы на 1 шаг вперед по таблице для выхода из цикла
        ,{STATE_10,     &StateMachine::Function6,       STEP_BACK_5}    // 5 шагов назад по таблице состояний к началу цикла
        ,{STATE_11,     &StateMachine::Function7,       STEP_FORWARD_1}
        ,{STATE_12,     &StateMachine::Function8,       STEP_FORWARD_1}
        ,{END_STATE,    &StateMachine::FunctionEnd,     STEP_FORWARD_1}
        ,{IDLE_STATE,   NULL,                           WAIT}
    };
    stateMachine->LoadTableProcess(tableStateInit, sizeof(tableStateInit));
}
//*********************************************************************************************************************
